/*************************************************************************
***** A Signal Framework for C++
**************************************************************************
                                                          
  Based on ideas from Gtk-- signal system by Tero Pulkkinen.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#ifndef SIGCXX_SIGCXX_H
#define SIGCXX_SIGCXX_H

#include <sigc++/slot.h>
#include <sigc++/signal.h>
#include <sigc++/bind.h>
#include <sigc++/bind_return.h>
#include <sigc++/hide.h>
#include <sigc++/method_slot.h>
#include <sigc++/object.h>
#include <sigc++/object_slot.h>

#endif /* SIGCXX_SIGCXX_H */

